@@ -5,7 +5,7 @@ use Params::Validate ();
 use Params::CallbackRequest::Exceptions (abbr => [qw(throw_bad_params)]);
 
 use vars qw($VERSION);
-$VERSION = '1.14';
+$VERSION = '1.16';
 use constant DEFAULT_PRIORITY => 5;
 use constant REDIRECT => 302;
 
@@ -17,12 +17,12 @@ Params::Validate::validation_options
 
 my $is_num = { 'valid priority' => sub { $_[0] =~ /^\d$/ } };
 
-# Use Apache::RequestRec for mod_perl 2
-my $ap_req_class = defined $mod_perl::VERSION && $mod_perl::VERSION >= 1.99
-    ? 'Apache::RequestRec'
-    : defined $mod_perl2::VERSION
+# Use Apache2?::RequestRec for mod_perl 2
+use constant APREQ_CLASS => exists $ENV{MOD_PERL_API_VERSION}
+    ? $ENV{MOD_PERL_API_VERSION} >= 2
         ? 'Apache2::RequestRec'
-        : 'Apache';
+        : 'Apache::RequestRec'
+    : 'Apache';
 
 BEGIN {
     # The object-oriented interface is only supported with the use of
@@ -42,6 +42,7 @@ BEGIN {
                       priority
                       cb_key
                       pkg_key
+                      requester
                       trigger_key
                       value )) {
         no strict 'refs';
@@ -50,50 +51,53 @@ BEGIN {
     *class_key = \&pkg_key;
 }
 
-my %valid_params =
-  ( cb_request      =>
-    { isa        => 'Params::CallbackRequest',
+my %valid_params = (
+    cb_request => { isa => 'Params::CallbackRequest' },
+
+    params => {
+        type => Params::Validate::HASHREF,
     },
 
-    params =>
-    {  type      => Params::Validate::HASHREF,
+    apache_req => {
+        isa      => APREQ_CLASS,
+        optional => 1,
     },
 
-    apache_req   =>
-    { isa        => $ap_req_class,
-      optional   => 1,
+    priority => {
+        type      => Params::Validate::SCALAR,
+        callbacks => $is_num,
+        optional  => 1,
+        desc      => 'Priority'
     },
 
-    priority =>
-    { type      => Params::Validate::SCALAR,
-      callbacks => $is_num,
-      optional  => 1,
-      desc      => 'Priority'
+    cb_key => {
+        type     => Params::Validate::SCALAR,
+        optional => 1,
+        desc     => 'Callback key'
     },
 
-    cb_key =>
-    { type      => Params::Validate::SCALAR,
-      optional  => 1,
-      desc      => 'Callback key'
+    pkg_key => {
+        type     => Params::Validate::SCALAR,
+        optional => 1,
+        desc     => 'Package key'
     },
 
-    pkg_key =>
-    { type      => Params::Validate::SCALAR,
-      optional  => 1,
-      desc      => 'Package key'
+    trigger_key => {
+        type     => Params::Validate::SCALAR,
+        optional => 1,
+        desc     => 'Trigger key'
     },
 
-    trigger_key =>
-    { type      => Params::Validate::SCALAR,
-      optional  => 1,
-      desc      => 'Trigger key'
+    value => {
+        optional => 1,
+        desc     => 'Callback value'
     },
 
-    value =>
-    { optional  => 1,
-      desc      => 'Callback value'
-    },
-  );
+    requester => {
+        optional => 1,
+        desc     => 'Requesting object'
+    }
+);
 
 sub new {
     my $proto = shift;
@@ -350,7 +354,7 @@ sub redirect {
     if (my $r = $self->apache_req) {
         $r->method('GET');
         $r->headers_in->unset('Content-length');
-        $r->err_header_out( Location => $url );
+        $r->err_headers_out->add( Location => $url );
     }
     $self->abort($status) unless $wait;
 }
@@ -481,6 +485,15 @@ passed one to C<< Params::CallbackRequest->request >>. This will be most
 useful in a mod_perl environment, of course. Use Apache:FakeRequest in
 tests to emmulate the behavior of an Apache request object.
 
+=head3 requester
+
+  my $r = $cb->requester;
+
+Returns the object that executed the callback by calling C<request()> on a
+Params::CallbackRequest object. Only available if the C<requester> parameter
+is passed to C<< Params::CallbackRequest->request >>. This can be useful for
+callbacks to get access to the object that executed the callbacks.
+
 =head3 priority
 
   my $priority = $cb->priority;
